// License: GPL. Copyright 2007-2008 by Brett Henderson and other contributors.
package org.openstreetmap.osmosis.core.apidb.v0_6.impl;

import java.sql.ResultSet;
import java.sql.SQLException;

import org.openstreetmap.osmosis.core.OsmosisRuntimeException;
import org.openstreetmap.osmosis.core.apidb.common.BaseTableReader;
import org.openstreetmap.osmosis.core.apidb.common.DatabaseContext;
import org.openstreetmap.osmosis.core.database.DatabaseLoginCredentials;
import org.openstreetmap.osmosis.core.domain.v0_6.WayNode;

/**
 * Reads all way nodes from a database ordered by the way identifier but not by the sequence.
 * 
 * @author Brett Henderson
 */
public class WayNodeTableReader extends BaseTableReader<DbFeatureHistory<DbOrderedFeature<WayNode>>> {

    private static final String SELECT_SQL = "SELECT id as way_id, version, node_id, sequence_id" + " FROM way_nodes"
            + " ORDER BY id, version";

    /**
     * Creates a new instance.
     * 
     * @param loginCredentials Contains all information required to connect to the database.
     */
    public WayNodeTableReader(DatabaseLoginCredentials loginCredentials) {
        super(loginCredentials);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected ResultSet createResultSet(DatabaseContext queryDbCtx) {
        return queryDbCtx.executeQuery(SELECT_SQL);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected ReadResult<DbFeatureHistory<DbOrderedFeature<WayNode>>> createNextValue(ResultSet resultSet) {
        long wayId;
        long nodeId;
        int sequenceId;
        int version;

        try {
            wayId = resultSet.getLong("way_id");
            nodeId = resultSet.getLong("node_id");
            sequenceId = resultSet.getInt("sequence_id");
            version = resultSet.getInt("version");

        } catch (SQLException e) {
            throw new OsmosisRuntimeException("Unable to read way node fields.", e);
        }

        return new ReadResult<DbFeatureHistory<DbOrderedFeature<WayNode>>>(true,
                new DbFeatureHistory<DbOrderedFeature<WayNode>>(new DbOrderedFeature<WayNode>(wayId,
                        new WayNode(nodeId), sequenceId), version));
    }
}
